--[[
##########################################################
S V U I   By: Munglunch
########################################################## 
LOCALIZED LUA FUNCTIONS
##########################################################
]]--
--[[ GLOBALS ]]--
local _G = _G;
local unpack    = _G.unpack;
local select    = _G.select;
local pairs     = _G.pairs;
local ipairs    = _G.ipairs;
local type      = _G.type;
local error     = _G.error;
local pcall     = _G.pcall;
local tostring  = _G.tostring;
local tonumber  = _G.tonumber;
local tinsert 	= _G.tinsert;
local string 	= _G.string;
local math 		= _G.math;
local table 	= _G.table;
--[[ STRING METHODS ]]--
local format = string.format;
--[[ MATH METHODS ]]--
local abs, ceil, floor, round = math.abs, math.ceil, math.floor, math.round;
--[[ TABLE METHODS ]]--
local tremove, twipe = table.remove, table.wipe;
--BLIZZARD API
local CreateFrame           = _G.CreateFrame;
local InCombatLockdown      = _G.InCombatLockdown;
local GameTooltip           = _G.GameTooltip;
local ReloadUI              = _G.ReloadUI;
local hooksecurefunc        = _G.hooksecurefunc;
local GetTime               = _G.GetTime;
local UnitName              = _G.UnitName;
local UnitGUID              = _G.UnitGUID;
local UnitAura              = _G.UnitAura;
local UnitBuff              = _G.UnitBuff;
local UnitDebuff            = _G.UnitDebuff;
local UnitStat              = _G.UnitStat;
local UnitLevel             = _G.UnitLevel;
local UnitClass             = _G.UnitClass;
local NUM_LE_RAID_BUFF_TYPES = _G.NUM_LE_RAID_BUFF_TYPES;
local RAID_CLASS_COLORS     = _G.RAID_CLASS_COLORS;
local CUSTOM_CLASS_COLORS   = _G.CUSTOM_CLASS_COLORS;
local GetItemQualityColor   = _G.GetItemQualityColor;
local GetInventoryItemQuality   = _G.GetInventoryItemQuality;
local GetInventoryItemTexture   = _G.GetInventoryItemTexture;
local GetWeaponEnchantInfo  = _G.GetWeaponEnchantInfo;
local RegisterStateDriver   = _G.RegisterStateDriver;
local UnregisterStateDriver = _G.UnregisterStateDriver;
local RegisterAttributeDriver   = _G.RegisterAttributeDriver;
local GetRaidBuffTrayAuraInfo 	= _G.GetRaidBuffTrayAuraInfo;
--[[ 
########################################################## 
GET ADDON DATA
##########################################################
]]--
local SV = _G['SVUI']
local L = SV.L
local MOD = SV.Auras;
if(not MOD) then return end;
--[[ 
########################################################## 
LOCAL VARS
##########################################################
]]--
local GUIDs = {};
local RunningProcs = {};
local ProcWatch = CreateFrame('Frame', 'SVUI_ProcWatch', UIParent);

local function CreateProcIcon()
	local proc = CreateFrame("Frame", nil, ProcWatch)
	proc:SetBackdrop({
		bgFile = [[Interface\BUTTONS\WHITE8X8]], 
		tile = false, 
		tileSize = 0, 
		edgeFile = [[Interface\BUTTONS\WHITE8X8]], 
		edgeSize = 2, 
		insets = {
			left = 0, 
			right = 0, 
			top = 0, 
			bottom = 0
		}
	 })
 	proc:SetBackdropColor(0, 0, 0)
 	proc:SetBackdropBorderColor(0, 0, 0)
	
	proc.icon = proc:CreateTexture(nil, "BORDER")
	proc.icon:InsetPoints(proc, 2, 2)
	proc.icon:SetTexCoord(unpack(_G.SVUI_ICON_COORDS))

	proc.count = proc:CreateFontString(nil, "ARTWORK")
	proc.count:SetPoint("BOTTOMRIGHT", (-1 + SV.db.Auras.countOffsetH), (1 + SV.db.Auras.countOffsetV))
	proc.count:SetFontObject(SVUI_Font_Aura)

	proc.time = proc:CreateFontString(nil, "ARTWORK")
	proc.time:SetPoint("TOP", proc, "BOTTOM", 1 + SV.db.Auras.timeOffsetH, 0 + SV.db.Auras.timeOffsetV)
	proc.time:SetFontObject(SVUI_Font_Aura)

	proc.cd = CreateFrame("Cooldown", nil, proc)
	proc.cd:SetAllPoints(proc)
	proc.cd:SetFrameLevel(proc:GetFrameLevel() + 10)
	--proc.cd:SetReverse(true)
	proc.cd:SetDrawEdge(false)
    proc.cd:SetDrawBling(false)
    proc.cd:SetSwipeColor(0, 0, 0, 1)
	proc.cd:SetAlpha(1)

	proc.highlight = proc:CreateTexture(nil, "HIGHLIGHT")
	proc.highlight:SetTexture(SV.media.statusbar.default)
	proc.highlight:SetVertexColor(1, 1, 1, 0.45)
	proc.highlight:InsetPoints(proc, 2, 2)

	SV.Animate:Flash(proc)

	return proc;
end
--[[ 
########################################################## 
CORE FUNCTIONS
##########################################################
]]--
local setupGUID
do 
	local cache = setmetatable({}, {__type = "k"})

	local frame = CreateFrame"Frame"
	frame:SetScript("OnEvent", function(self, event)
		for k,t in pairs(GUIDs) do
			GUIDs[k] = nil
			for a in pairs(t) do
				t[a] = nil
			end
			cache[t] = true
		end
	end)
	frame:RegisterEvent"PLAYER_REGEN_ENABLED"
	frame:RegisterEvent"PLAYER_ENTERING_WORLD"
	
	function setupGUID(guid)
		local t = next(cache)
		if t then
			cache[t] = nil
		else
			t = {}
		end
		GUIDs[guid] = t
	end
end

local function SetProcTimer(icon, count, duration, remaining)
	icon:Show()
	if icon.cd then
		if duration and duration > 0 then
			icon.cd:SetCooldown(remaining - duration, duration)
		end
	end
	if icon.count then
		icon.count:SetText((count > 1 and count))
	end
end

local PROC_UNIT_AURA = function(self, event, unit)
	if not unit then return end
	local index, icons = 1, ProcWatch.watched
	local _, name, texture, count, duration, remaining, caster, key, aura, spellID
	local filter = "HELPFUL"
	local guid = UnitGUID(unit)
	if not guid then return end
	if not GUIDs[guid] then setupGUID(guid) end
	
	for _, icon in pairs(icons) do
		icon:Hide()
	end

	local x = 1
	while true do
		name, _, texture, count, _, duration, remaining, caster, _, _, spellID = UnitAura(unit, index, filter)
		if not name then 
			if filter == "HELPFUL" then
				filter = "HARMFUL"
				index = 1
			else
				break
			end
		else 
			if icons[spellID] then
				aura = icons[spellID]
				aura.timeLeft = remaining;
				aura.first = true;
				GUIDs[guid][spellID] = true;
				RunningProcs[spellID] = true;
				aura:ClearAllPoints()
				aura:SetPoint('RIGHT', ProcWatch, 'RIGHT', -((x - 1) * 43), 0)
				SetProcTimer(aura, count, duration, remaining);
				x = x + 1
			end
			index = index + 1
		end
	end
	
	for key in pairs(GUIDs[guid]) do
		if icons[key] and not RunningProcs[key] then
			icons[key]:Hide()
		end
	end
end

function MOD:UpdateProcWatch()
	if(not SV.db.Auras.procsEnabled) then return end;
	local PROCS = ProcWatch.cache or {};
	ProcWatch.watched = {}
	for _,aura in pairs(PROCS) do
	
		local name, _, image = GetSpellInfo(aura.spellID)

		if name then
			ProcWatch.watched[aura.spellID] = aura
		end
	end
end;

function MOD:InitializeProcWatch()
	if(not SV.db.Auras.procsEnabled) then return end;

	if(not SV.db.Filters.Procs) then
		SV.db.Filters.Procs = {}
	end

	local ProcsAnchor = CreateFrame('Frame', 'SVUI_ProcWatchFrame', SV.Screen)
	ProcsAnchor:SetSize(172,40)
	ProcsAnchor:SetPoint("TOPRIGHT",SV.Screen,"CENTER",-50,-50)

	ProcWatch:SetParent(ProcsAnchor)
	ProcWatch:SetWidth(720)
	ProcWatch:SetHeight(40)
	ProcWatch:SetPoint('RIGHT',ProcsAnchor,'RIGHT',0,0)
	ProcWatch.cache = {};
	ProcWatch.watched = {};
	ProcWatch:RegisterUnitEvent('UNIT_AURA',"player")
	ProcWatch:SetScript('OnEvent', PROC_UNIT_AURA)

	SV:NewAnchor(ProcsAnchor, L["Procs Frame"])

	local PROCS = ProcWatch.cache;
	local pwSize = SV.db.Auras.procSize or 40;
	--local list = SV.db.Filters.Procs
	local list = SV.db.Filters.BuffWatch
	local i = 1;
	for procID,procData in pairs(list) do
		local spellID = tonumber(procID);
		local spellName,_,spellTexture = GetSpellInfo(spellID)
		if spellName then
			local proc;
			if not PROCS[procID] then 
				proc = CreateProcIcon()
				PROCS[procID] = proc
			else 
				proc = PROCS[procID]
			end;

			proc.name = spellName;
			proc.index = i;
			proc.spellID = spellID;
			proc:SetWidth(pwSize)
			proc:SetHeight(pwSize)
			proc:ClearAllPoints()
			proc:SetPoint('RIGHT', ProcWatch, 'RIGHT', -((i - 1) * 43), 0)
			proc.icon:SetTexture(spellTexture)
			proc:Hide()

			ProcWatch.cache[procID] = proc;

			if ProcWatch.watched then 
				ProcWatch.watched[procID] = proc
			end
			i = i + 1
		end
	end;

	self:UpdateProcWatch()
end;